from __future__ import print_function

import os

from django import VERSION as DJANGO_VERSION

SECRET_KEY = 'w6bidenrf5q%byf-q82b%pli50i0qmweus6gt_3@k$=zg7ymd3'
SITE_ID = 1

INSTALLED_APPS = (
    'django.contrib.sessions',
    'django.contrib.contenttypes',
    'django.contrib.auth',
    'django.contrib.admin',
    'django.contrib.staticfiles',
    'django.contrib.sites',
    'tests',
    'authtools',
    'auth_tests',
)

MIDDLEWARE = [
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
]

if DJANGO_VERSION < (1, 10):
    MIDDLEWARE_CLASSES = MIDDLEWARE + [
        'django.contrib.auth.middleware.SessionAuthenticationMiddleware',
    ]


DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'sqlite_database',
    }
}

MIGRATION_MODULES = {
    'auth': None,
}

ROOT_URLCONF = 'tests.urls'

STATIC_URL = '/static/'
DEBUG = True

AUTH_USER_MODEL = os.environ.get('AUTH_USER_MODEL', 'auth.User')

print('Using %s as the AUTH_USER_MODEL.' % AUTH_USER_MODEL)


TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.contrib.auth.context_processors.auth',
            ],
        },
    },
]

# sha1 hasher was removed as a default in django 1.10. Need it for the tests.
PASSWORD_HASHERS = [
    'django.contrib.auth.hashers.SHA1PasswordHasher',
    'django.contrib.auth.hashers.PBKDF2PasswordHasher',
    'django.contrib.auth.hashers.PBKDF2SHA1PasswordHasher',
    'django.contrib.auth.hashers.BCryptSHA256PasswordHasher',
    'django.contrib.auth.hashers.BCryptPasswordHasher',
]



from __future__ import unicode_literals

from django.contrib.auth.models import AbstractBaseUser, BaseUserManager, PermissionsMixin
from django.core.mail import send_mail
from django.db import models
from django.utils import timezone
from six import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _


class UserManager(BaseUserManager):
    def create_user(self, email, password=None, **kwargs):
        email = self.normalize_email(email)
        user = self.model(email=email, **kwargs)
        user.set_password(password)
        user.save(using=self._db)
        return user

    def create_superuser(self, **kwargs):
        user = self.create_user(**kwargs)
        user.is_superuser = True
        user.is_staff = True
        user.save(using=self._db)
        return user


class AbstractEmailUser(AbstractBaseUser, PermissionsMixin):
    email = models.EmailField(_('email address'), max_length=255, unique=True)

    is_staff = models.BooleanField(_('staff status'), default=False,
        help_text=_('Designates whether the user can log into this admin '
                    'site.'))
    is_active = models.BooleanField(_('active'), default=True,
        help_text=_('Designates whether this user should be treated as '
                    'active. Unselect this instead of deleting accounts.'))
    date_joined = models.DateTimeField(_('date joined'), default=timezone.now)

    objects = UserManager()

    USERNAME_FIELD = 'email'
    REQUIRED_FIELDS = []

    class Meta:
        abstract = True
        ordering = ['email']

    def get_full_name(self):
        return self.email

    def get_short_name(self):
        return self.email

    def email_user(self, subject, message, from_email=None, **kwargs):
        """Sends an email to this User."""

        send_mail(subject, message, from_email, [self.email], **kwargs)

@python_2_unicode_compatible
class AbstractNamedUser(AbstractEmailUser):
    name = models.CharField(_('name'), max_length=255)

    REQUIRED_FIELDS = ['name']

    class Meta:
        abstract = True
        ordering = ['name', 'email']

    def __str__(self):
        return '{name} <{email}>'.format(
            name=self.name,
            email=self.email,
        )

    def get_full_name(self):
        return self.name

    def get_short_name(self):
        return self.name


class User(AbstractNamedUser):
    class Meta(AbstractNamedUser.Meta):
        swappable = 'AUTH_USER_MODEL'
        verbose_name = _('user')
        verbose_name_plural = _('users')



from django.conf.urls import url

from authtools import views


urlpatterns = [
    url(
        r'^login/$',
        views.LoginView.as_view(),
        name='login'
    ),
    url(
        r'^logout/$'
        , views.LogoutView.as_view(),
        name='logout'
    ),
    url(
        r'^password_change/$',
        views.PasswordChangeView.as_view(),
        name='password_change'
    ),
    url(
        r'^password_change/done/$',
        views.PasswordChangeDoneView.as_view(),
        name='password_change_done'
    ),
    url(
        r'^password_reset/$',
        views.PasswordResetView.as_view(),
        name='password_reset'
    ),
    url(
        r'^password_reset/done/$',
        views.PasswordResetDoneView.as_view(),
        name='password_reset_done'
    ),
    url(
        r'^reset/done/$',
        views.PasswordResetCompleteView.as_view(),
        name='password_reset_complete'
    ),
    url(
        r'^reset/(?P<uidb64>[0-9A-Za-z_\-]+)/(?P<token>[0-9A-Za-z]{1,13}-[0-9A-Za-z]{1,20})/$',
        views.PasswordResetConfirmView.as_view(),
        name='password_reset_confirm'
    ),
]



"""
Mostly equivalent to the views from django.contrib.auth.views, but
implemented as class-based views.
"""
from __future__ import unicode_literals
import warnings

from django.conf import settings
from django.contrib.auth import (
    get_user_model, update_session_auth_hash,
    REDIRECT_FIELD_NAME, login as auth_login
)
from django.contrib.auth.views import (
    SuccessURLAllowedHostsMixin,
    INTERNAL_RESET_SESSION_TOKEN
)
from django.contrib.auth.decorators import login_required
from django.contrib.auth.forms import (
    SetPasswordForm, PasswordChangeForm, PasswordResetForm
)
from django.contrib.auth.tokens import default_token_generator
from django.contrib import auth
from django.http import HttpResponseRedirect
from django.urls import reverse_lazy

from django.contrib.sites.shortcuts import get_current_site

from django.shortcuts import redirect, resolve_url
from django.utils.functional import lazy
from django.utils.http import base36_to_int, is_safe_url, urlsafe_base64_decode
import six
from django.views.decorators.cache import never_cache
from django.views.decorators.csrf import csrf_protect
from django.views.decorators.debug import sensitive_post_parameters
from django.views.generic import FormView, TemplateView, RedirectView
from django import VERSION as DJANGO_VERSION

from .forms import AuthenticationForm

User = get_user_model()


INTERNAL_RESET_URL_TOKEN = "set-password"


def _safe_resolve_url(url):
    """
    Previously, resolve_url_lazy would fail if the url was a unicode object.
    See <https://github.com/fusionbox/django-authtools/issues/13> for more
    information.

    Thanks to GitHub user alanwj for pointing out the problem and providing
    this solution.
    """
    return six.text_type(resolve_url(url))

resolve_url_lazy = lazy(_safe_resolve_url, six.text_type)


class WithCurrentSiteMixin(object):
    def get_current_site(self):
        return get_current_site(self.request)

    def get_context_data(self, **kwargs):
        kwargs = super(WithCurrentSiteMixin, self).get_context_data(**kwargs)
        current_site = self.get_current_site()
        kwargs.update({
            'site': current_site,
            'site_name': current_site.name,
        })
        return kwargs


class WithNextUrlMixin(object):
    redirect_field_name = REDIRECT_FIELD_NAME
    success_url = None

    def get_next_url(self):
        request = self.request
        redirect_to = request.POST.get(self.redirect_field_name,
                                       request.GET.get(self.redirect_field_name, ''))
        if not redirect_to:
            return

        host = self.request.get_host()
        allowed_hosts = [host]

        try:
            allowed_hosts += self.get_success_url_allowed_hosts()
        except AttributeError:
            pass

        url_is_safe = is_safe_url(
            redirect_to,
            allowed_hosts=allowed_hosts,
            require_https=self.request.is_secure()
        )

        if url_is_safe:
            return redirect_to

    # This mixin can be mixed with FormViews and RedirectViews. They
    # each use a different method to get the URL to redirect to, so we
    # need to provide both methods.
    def get_success_url(self):
        return self.get_next_url() or super(WithNextUrlMixin, self).get_success_url()

    def get_redirect_url(self, **kwargs):
        return self.get_next_url() or super(WithNextUrlMixin, self).get_redirect_url(**kwargs)


def DecoratorMixin(decorator):
    """
    Converts a decorator written for a function view into a mixin for a
    class-based view.

    ::

        LoginRequiredMixin = DecoratorMixin(login_required)

        class MyView(LoginRequiredMixin):
            pass

        class SomeView(DecoratorMixin(some_decorator),
                       DecoratorMixin(something_else)):
            pass

    """

    class Mixin(object):
        __doc__ = decorator.__doc__

        @classmethod
        def as_view(cls, *args, **kwargs):
            view = super(Mixin, cls).as_view(*args, **kwargs)
            return decorator(view)

    Mixin.__name__ = str('DecoratorMixin(%s)' % decorator.__name__)
    return Mixin


NeverCacheMixin = DecoratorMixin(never_cache)
CsrfProtectMixin = DecoratorMixin(csrf_protect)
LoginRequiredMixin = DecoratorMixin(login_required)
SensitivePostParametersMixin = DecoratorMixin(
    sensitive_post_parameters('password', 'old_password', 'password1',
                              'password2', 'new_password1', 'new_password2')
)

class AuthDecoratorsMixin(NeverCacheMixin, CsrfProtectMixin, SensitivePostParametersMixin):
    pass


class LoginView(AuthDecoratorsMixin, SuccessURLAllowedHostsMixin,
                WithCurrentSiteMixin, WithNextUrlMixin, FormView):
    form_class = AuthenticationForm
    authentication_form = None
    template_name = 'registration/login.html'
    allow_authenticated = True
    success_url = resolve_url_lazy(settings.LOGIN_REDIRECT_URL)

    # BBB: This is deprecated (See LoginView.get_allow_authenticated)
    disallow_authenticated = None

    def get_allow_authenticated(self):
        if self.disallow_authenticated is not None:
            warnings.warn("disallow_authenticated is deprecated. Please use allow_authenticated",
                          DeprecationWarning)
            return not self.disallow_authenticated
        else:
            return self.allow_authenticated

    def dispatch(self, *args, **kwargs):
        allow_authenticated = self.get_allow_authenticated()
        if not allow_authenticated and self.request.user.is_authenticated:
            return redirect(self.get_success_url())
        return super(LoginView, self).dispatch(*args, **kwargs)

    def get_form_class(self):
        return self.authentication_form or self.form_class

    def form_valid(self, form):
        auth.login(self.request, form.get_user())
        return super(LoginView, self).form_valid(form)

    def get_context_data(self, **kwargs):
        kwargs = super(LoginView, self).get_context_data(**kwargs)
        kwargs.update({
            self.redirect_field_name: self.request.GET.get(
                self.redirect_field_name, '',
            ),
        })
        return kwargs

    def get_form_kwargs(self):
        kwargs = super(LoginView, self).get_form_kwargs()
        kwargs['request'] = self.request
        return kwargs


class LogoutView(NeverCacheMixin, SuccessURLAllowedHostsMixin, WithCurrentSiteMixin, WithNextUrlMixin, TemplateView,
                 RedirectView):
    template_name = 'registration/logged_out.html'
    permanent = False

    def get_redirect_url(self, **kwargs):
        redirect_to = super(LogoutView, self).get_redirect_url(**kwargs)

        if redirect_to:
            return redirect_to
        elif settings.LOGOUT_REDIRECT_URL is not None:
            return resolve_url(settings.LOGOUT_REDIRECT_URL)
        elif self.request.POST.get(self.redirect_field_name, self.request.GET.get(self.redirect_field_name, '')):
            # we have a url, but it is not safe. Django redirects back to the same view.
            return self.request.path

    def get(self, *args, **kwargs):
        auth.logout(self.request)
        # If we have a url to redirect to, do it. Otherwise render the logged-out template.
        if self.get_redirect_url(**kwargs):
            return RedirectView.get(self, *args, **kwargs)
        else:
            return TemplateView.get(self, *args, **kwargs)


class PasswordChangeView(LoginRequiredMixin, WithNextUrlMixin, AuthDecoratorsMixin, FormView):
    template_name = 'registration/password_change_form.html'
    form_class = PasswordChangeForm
    success_url = reverse_lazy('password_change_done')

    def get_form_kwargs(self):
        kwargs = super(PasswordChangeView, self).get_form_kwargs()
        kwargs['user'] = self.get_user()
        return kwargs

    def get_user(self):
        return self.request.user

    def form_valid(self, form):
        form.save()
        # Updating the password logs out all other sessions for the user
        # except the current one if
        # django.contrib.auth.middleware.SessionAuthenticationMiddleware
        # is enabled.
        update_session_auth_hash(self.request, form.user)
        return super(PasswordChangeView, self).form_valid(form)


class PasswordChangeDoneView(LoginRequiredMixin, TemplateView):
    template_name = 'registration/password_change_done.html'


# 4 views for password reset:
# - PasswordResetView sends the mail
# - PasswordResetDoneView shows a success message for the above
# - PasswordResetConfirmView checks the link the user clicked and
#   prompts for a new password
# - PasswordResetCompleteView shows a success message for the above


class PasswordResetView(CsrfProtectMixin, FormView):
    template_name = 'registration/password_reset_form.html'
    token_generator = default_token_generator
    success_url = reverse_lazy('password_reset_done')
    domain_override = None
    subject_template_name = 'registration/password_reset_subject.txt'
    email_template_name = 'registration/password_reset_email.html'
    html_email_template_name = None
    from_email = None
    form_class = PasswordResetForm
    extra_email_context = None

    def form_valid(self, form):
        kwargs = dict(
            domain_override=self.domain_override,
            subject_template_name=self.subject_template_name,
            email_template_name=self.email_template_name,
            token_generator=self.token_generator,
            from_email=self.from_email,
            request=self.request,
            use_https=self.request.is_secure(),
            html_email_template_name=self.html_email_template_name,
        )

        if DJANGO_VERSION[:2] >= (1, 9):
            kwargs['extra_email_context'] = self.extra_email_context

        form.save(**kwargs)

        return super(PasswordResetView, self).form_valid(form)


class PasswordResetDoneView(TemplateView):
    template_name = 'registration/password_reset_done.html'


class PasswordResetConfirmView(AuthDecoratorsMixin, FormView):
    template_name = 'registration/password_reset_confirm.html'
    token_generator = default_token_generator
    form_class = SetPasswordForm
    success_url = reverse_lazy('password_reset_complete')
    post_reset_login = False
    post_reset_login_backend = None

    def dispatch(self, *args, **kwargs):
        assert self.kwargs.get('token') is not None
        self.user = self.get_user()
        self.validlink = False

        if self.user is not None:
            # Most of this is copied from django
            token = kwargs['token']
            if token == INTERNAL_RESET_URL_TOKEN:
                session_token = self.request.session.get(INTERNAL_RESET_SESSION_TOKEN)
                if self.token_generator.check_token(self.user, session_token):
                    # If the token is valid, display the password reset form.
                    self.validlink = True
                    return super(PasswordResetConfirmView, self).dispatch(*args, **kwargs)
            else:
                if self.token_generator.check_token(self.user, token):
                    # Store the token in the session and redirect to the
                    # password reset form at a URL without the token. That
                    # avoids the possibility of leaking the token in the
                    # HTTP Referer header.
                    self.request.session[INTERNAL_RESET_SESSION_TOKEN] = token
                    redirect_url = self.request.path.replace(token, INTERNAL_RESET_URL_TOKEN)
                    return HttpResponseRedirect(redirect_url)

        return super(PasswordResetConfirmView, self).dispatch(*args, **kwargs)

    def get_queryset(self):
        return User._default_manager.all()

    def get_user(self):
        uidb64 = self.kwargs.get('uidb64')
        try:
            uid = urlsafe_base64_decode(uidb64)
            return self.get_queryset().get(pk=uid)
        except (TypeError, ValueError, OverflowError, User.DoesNotExist):
            return None

    def valid_link(self):
        user = self.user
        return user is not None and self.token_generator.check_token(user, self.kwargs.get('token'))

    def get_form_kwargs(self):
        kwargs = super(PasswordResetConfirmView, self).get_form_kwargs()
        kwargs['user'] = self.user
        return kwargs

    def get_context_data(self, **kwargs):
        kwargs = super(PasswordResetConfirmView, self).get_context_data(**kwargs)
        if self.validlink:
            kwargs['validlink'] = True
        else:
            kwargs['validlink'] = False
            kwargs['form'] = None
        return kwargs

    def form_valid(self, form):
        if not self.validlink:
            return self.form_invalid(form)

        user = self.save_form(form)

        if INTERNAL_RESET_SESSION_TOKEN:
            del self.request.session[INTERNAL_RESET_SESSION_TOKEN]

        if self.post_reset_login:
            auth_login(self.request, user, self.post_reset_login_backend)

        return super(PasswordResetConfirmView, self).form_valid(form)

    def save_form(self, form):
        return form.save()


class PasswordResetConfirmAndLoginView(PasswordResetConfirmView):
    success_url = resolve_url_lazy(settings.LOGIN_REDIRECT_URL)

    def save_form(self, form):
        ret = super(PasswordResetConfirmAndLoginView, self).save_form(form)
        user = auth.authenticate(request=self.request,
                                 username=self.user.get_username(),
                                 password=form.cleaned_data['new_password1'])

        # post_reset_login will log the user in. We don't
        # need to do it here. But we do have to set the backend.
        self.post_reset_login = True
        self.post_reset_login_backend = user.backend

        return ret


class PasswordResetCompleteView(TemplateView):
    template_name = 'registration/password_reset_complete.html'
    login_url = settings.LOGIN_URL

    def get_login_url(self):
        return resolve_url(self.login_url)

    def get_context_data(self, **kwargs):
        kwargs = super(PasswordResetCompleteView, self).get_context_data(**kwargs)
        kwargs['login_url'] = self.get_login_url()
        return kwargs



"""
We're able to borrow most of django's auth view tests.

"""
import collections
import contextlib
import itertools
import warnings

import datetime

from unittest import skipIf, skipUnless

from django.core import mail
from django.urls import reverse

from django.contrib.auth import REDIRECT_FIELD_NAME, get_user_model, SESSION_KEY
from django.contrib.auth.forms import AuthenticationForm
from django.utils.http import urlquote
from django.test import TestCase
from django.test.client import RequestFactory
from django.test.utils import override_settings
from django.utils.encoding import force_text
from django.utils.translation import ugettext as _
from django.forms.fields import Field
from django.conf import settings
from django.http import HttpRequest
from django.middleware.csrf import CsrfViewMiddleware, get_token
from django.contrib.sessions.middleware import SessionMiddleware
from django import VERSION as DJANGO_VERSION

from django.contrib.auth.views import LoginView
login_view = LoginView.as_view()

from authtools.views import LoginView

from auth_tests.test_views import (
    AuthViewNamedURLTests,
    PasswordResetTest,
    ChangePasswordTest,
    LoginTest,
    LoginURLSettings,
    LogoutTest,
    LoginSuccessURLAllowedHostsTest,
)

from django.contrib.sites.shortcuts import get_current_site


from authtools.admin import BASE_FIELDS
from authtools.forms import (
    UserCreationForm,
    UserChangeForm,
    FriendlyPasswordResetForm,
    CaseInsensitiveUsernameFieldCreationForm,
    CaseInsensitiveEmailUserCreationForm,
)
from authtools.views import PasswordResetCompleteView, resolve_url_lazy

User = get_user_model()


def skipIfNotCustomUser(test_func):
    return skipIf(settings.AUTH_USER_MODEL == 'auth.User', 'Built-in User model in use')(test_func)


def skipIfCustomUser(test_func):
    """
    Copied from django.contrib.auth.tests.utils, This is deprecated in the future, but we still
    need it for some of our tests.
    """
    return skipIf(settings.AUTH_USER_MODEL != 'auth.User', 'Custom user model in use')(test_func)


class WarningTestMixin(object):
    @contextlib.contextmanager
    def assertWarns(self, warning_classes):
        if not isinstance(warning_classes, collections.Iterable):
            warning_classes = [warning_classes]

        with warnings.catch_warnings(record=True) as warn:
            warnings.simplefilter("always")
            yield
            assert len(warn) == len(warning_classes)
            for msg, expected_class in zip(warn, warning_classes):
                assert issubclass(msg.category, expected_class)


class EmailLoginMixin(object):
    if settings.AUTH_USER_MODEL != 'auth.User':
        if DJANGO_VERSION[:2] < (1, 9):
            if settings.AUTH_USER_MODEL == 'authtools.User':
                fixtures = ['authtoolstestdata.json']
            elif settings.AUTH_USER_MODEL == 'tests.User':
                fixtures = ['customusertestdata.json']

        def login(self, username='testclient', password='password'):
            """
            Authtools uses email addresses to login.

            Fortunately, email addresses in the fixtures are username + '@example.com'
            """
            if username == 'staff':
                username = 'staffmember@example.com'
            elif '@' not in username:
                username = username + '@example.com'
            return super(EmailLoginMixin, self).login(username, password)


class AuthViewsTestCaseDataMixin(object):
    """
    Helper base class for all the follow test cases.
    """

    if DJANGO_VERSION[:2] >= (1, 9):
        @classmethod
        def setUpTestData(cls):
            if settings.AUTH_USER_MODEL == 'auth.User':
                cls.u1 = User.objects.create(
                    password='sha1$6efc0$f93efe9fd7542f25a7be94871ea45aa95de57161',
                    last_login=datetime.datetime(2006, 12, 17, 7, 3, 31), is_superuser=False, username='testclient',
                    first_name='Test', last_name='Client', email='testclient@example.com', is_staff=False, is_active=True,
                    date_joined=datetime.datetime(2006, 12, 17, 7, 3, 31)
                )
                cls.u2 = User.objects.create(
                    password='sha1$6efc0$f93efe9fd7542f25a7be94871ea45aa95de57161',
                    last_login=datetime.datetime(2006, 12, 17, 7, 3, 31), is_superuser=False, username='inactive',
                    first_name='Inactive', last_name='User', email='testclient2@example.com', is_staff=False, is_active=False,
                    date_joined=datetime.datetime(2006, 12, 17, 7, 3, 31)
                )
                cls.u3 = User.objects.create(
                    password='sha1$6efc0$f93efe9fd7542f25a7be94871ea45aa95de57161',
                    last_login=datetime.datetime(2006, 12, 17, 7, 3, 31), is_superuser=False, username='staff',
                    first_name='Staff', last_name='Member', email='staffmember@example.com', is_staff=True, is_active=True,
                    date_joined=datetime.datetime(2006, 12, 17, 7, 3, 31)
                )
                cls.u4 = User.objects.create(
                    password='', last_login=datetime.datetime(2006, 12, 17, 7, 3, 31), is_superuser=False,
                    username='empty_password', first_name='Empty', last_name='Password', email='empty_password@example.com',
                    is_staff=False, is_active=True, date_joined=datetime.datetime(2006, 12, 17, 7, 3, 31)
                )
                cls.u5 = User.objects.create(
                    password='$', last_login=datetime.datetime(2006, 12, 17, 7, 3, 31), is_superuser=False,
                    username='unmanageable_password', first_name='Unmanageable', last_name='Password',
                    email='unmanageable_password@example.com', is_staff=False, is_active=True,
                    date_joined=datetime.datetime(2006, 12, 17, 7, 3, 31)
                )
                cls.u6 = User.objects.create(
                    password='foo$bar', last_login=datetime.datetime(2006, 12, 17, 7, 3, 31), is_superuser=False,
                    username='unknown_password', first_name='Unknown', last_name='Password',
                    email='unknown_password@example.com', is_staff=False, is_active=True,
                    date_joined=datetime.datetime(2006, 12, 17, 7, 3, 31)
                )
            else:
                cls.u1 = User.objects.create(
                    password='sha1$6efc0$f93efe9fd7542f25a7be94871ea45aa95de57161',
                    last_login=datetime.datetime(2006, 12, 17, 7, 3, 31), is_superuser=False,
                    email='testclient@example.com', is_staff=False, is_active=True,
                    date_joined=datetime.datetime(2006, 12, 17, 7, 3, 31)
                )
                cls.u2 = User.objects.create(
                    password='sha1$6efc0$f93efe9fd7542f25a7be94871ea45aa95de57161',
                    last_login=datetime.datetime(2006, 12, 17, 7, 3, 31), is_superuser=False,
                    email='testclient2@example.com', is_staff=False, is_active=False,
                    date_joined=datetime.datetime(2006, 12, 17, 7, 3, 31)
                )
                cls.u3 = User.objects.create(
                    password='sha1$6efc0$f93efe9fd7542f25a7be94871ea45aa95de57161',
                    last_login=datetime.datetime(2006, 12, 17, 7, 3, 31), is_superuser=False,
                    email='staffmember@example.com', is_staff=True, is_active=True,
                    date_joined=datetime.datetime(2006, 12, 17, 7, 3, 31)
                )
                cls.u4 = User.objects.create(
                    password='', last_login=datetime.datetime(2006, 12, 17, 7, 3, 31), is_superuser=False,
                    email='empty_password@example.com',
                    is_staff=False, is_active=True, date_joined=datetime.datetime(2006, 12, 17, 7, 3, 31)
                )
                cls.u5 = User.objects.create(
                    password='$', last_login=datetime.datetime(2006, 12, 17, 7, 3, 31), is_superuser=False,
                    email='unmanageable_password@example.com', is_staff=False, is_active=True,
                    date_joined=datetime.datetime(2006, 12, 17, 7, 3, 31)
                )
                cls.u6 = User.objects.create(
                    password='foo$bar', last_login=datetime.datetime(2006, 12, 17, 7, 3, 31), is_superuser=False,
                    email='unknown_password@example.com', is_staff=False, is_active=True,
                    date_joined=datetime.datetime(2006, 12, 17, 7, 3, 31)
                )


@override_settings(ROOT_URLCONF='authtools.urls')
class AuthViewNamedURLTests(AuthViewsTestCaseDataMixin, AuthViewNamedURLTests):
    if DJANGO_VERSION[:2] < (1, 9):
        if settings.AUTH_USER_MODEL == 'authtools.User':
            fixtures = ['authtoolstestdata.json']
        elif settings.AUTH_USER_MODEL == 'tests.User':
            fixtures = ['customusertestdata.json']


class UtilsTest(TestCase):
    def test_resolve_lazy_unicode(self):
        self.assertTrue(resolve_url_lazy(u'/'))


@override_settings(ROOT_URLCONF='tests.urls')
class PasswordResetTest(EmailLoginMixin, AuthViewsTestCaseDataMixin, PasswordResetTest):
    # these use custom, test-specific urlpatterns that we don't have
    test_admin_reset = None
    test_reset_custom_redirect = None
    test_reset_custom_redirect_named = None
    test_email_found_custom_from = None
    test_confirm_redirect_custom = None
    test_confirm_redirect_custom_named = None
    # these reference the builtin user model
    test_confirm_invalid_post = None
    test_confirm_display_user_from_form = None
    test_confirm_complete = None

    def assertFormError(self, response, error):
        """Assert that error is found in response.context['form'] errors"""
        form_errors = list(itertools.chain(*response.context['form'].errors.values()))
        self.assertIn(force_text(error), form_errors)

    # test the django 1.5 behavior
    def test_email_not_found_in_friendly_password_reset_form(self):
        "Error is raised if the provided email address isn't currently registered"
        response = self.client.get('/friendly_password_reset/')
        self.assertEqual(response.status_code, 200)
        response = self.client.post('/friendly_password_reset/',
                                    {'email': 'not_a_real_email@email.com'})
        self.assertFormError(response, FriendlyPasswordResetForm.error_messages['unknown'])
        self.assertEqual(len(mail.outbox), 0)

    def test_user_only_fetched_once(self):
        url, confirm_path = self._test_confirm_start()
        num_queries = 9

        with self.assertNumQueries(num_queries):
            # the confirm view is only allowed to fetch the user object a
            # single time
            self.client.get(confirm_path)

    def test_confirm_invalid_path(self):
        # django has a similar test, but it tries to test an invalid path AND
        # an invalid form at the same time. We need a test case with an invalid
        # path, but valid form.
        url, path = self._test_confirm_start()
        path = path[:-5] + ("0" * 4) + path[-1]

        self.client.post(path, {
            'new_password1': 'anewpassword',
            'new_password2': 'anewpassword',
        })
        # Check the password has not been changed
        u = User.objects.get(email='staffmember@example.com')
        self.assertTrue(not u.check_password("anewpassword"))

    def test_confirm_done(self):
        """
        Password reset complete page should be rendered with 'login_url'
        in its context.
        """
        url, path = self._test_confirm_start()
        response = self.client.post(path, {'new_password1': 'anewpassword',
                                           'new_password2': 'anewpassword'})
        self.assertEqual(response.status_code, 302)

        response = self.client.get(response['Location'])

        self.assertIn('login_url', response.context)

    def test_confirm_login_url_resolves(self):
        complete_view = PasswordResetCompleteView.as_view(login_url='login_required')
        request_factory = RequestFactory()
        response = complete_view(request_factory.get('/xxx/'))
        self.assertEqual(response.context_data['login_url'], reverse('login_required'))

        complete_view2 = PasswordResetCompleteView.as_view(login_url='/dont-change-me/')
        response = complete_view2(request_factory.get('/xxx/'))
        self.assertEqual(response.context_data['login_url'], '/dont-change-me/')

    def test_confirm_and_login(self):
        url, path = self._test_confirm_start()
        path = path.replace('reset', 'reset_and_login')
        response = self.client.post(path, {'new_password1': 'anewpassword',
                                           'new_password2': 'anewpassword'})
        self.assertEqual(response.status_code, 302)


        response = self.client.post(response.url, {'new_password1': 'anewpassword',
                                            'new_password2': 'anewpassword'})
        self.assertEqual(response.status_code, 302)

        # verify that we're actually logged in
        response = self.client.get('/login_required/')
        self.assertEqual(response.status_code, 200)

    def test_confirm_invalid_hash(self):
        """A POST with an invalid token is rejected."""
        u = User.objects.get(email='staffmember@example.com')
        original_password = u.password
        url, path = self._test_confirm_start()
        path_parts = path.split('-')
        path_parts[-1] = ("0") * 20 + '/'
        path = '-'.join(path_parts)

        response = self.client.post(path, {
            'new_password1': 'anewpassword',
            'new_password2': 'anewpassword',
        })
        self.assertIs(response.context['validlink'], False)
        u.refresh_from_db()
        self.assertEqual(original_password, u.password)  # password hasn't changed


@override_settings(ROOT_URLCONF='authtools.urls')
class ChangePasswordTest(EmailLoginMixin, AuthViewsTestCaseDataMixin, ChangePasswordTest):

    test_password_change_redirect_custom = None
    test_password_change_redirect_custom_named = None

    # the builtin test doesn't logout after the password is changed, so
    # fail_login doesn't do anything when disallow_authenticated is True.
    def test_password_change_succeeds(self):
        self.login()
        self.client.post('/password_change/', {
            'old_password': 'password',
            'new_password1': 'password1',
            'new_password2': 'password1',
        })
        self.logout()
        self.fail_login()
        self.login(password='password1')

    def fail_login(self, password='password'):
        response = self.client.post('/login/', {
            'username': 'testclient',
            'password': password,
        })
        self.assertFormError(response, AuthenticationForm.error_messages['invalid_login'] % {
            'username': User._meta.get_field(User.USERNAME_FIELD).verbose_name
        })


# @override_settings(ROOT_URLCONF='authtools.urls')
class LoginTest(EmailLoginMixin, AuthViewsTestCaseDataMixin, LoginTest):
    # the built-in tests depend on the django urlpatterns (they reverse
    # django.contrib.auth.views.login)

    if settings.AUTH_USER_MODEL == 'auth.User':
        default_login = 'testclient'
    else:
        default_login = 'testclient@example.com'

    def test_current_site_in_context_after_login(self):
        response = self.client.get(reverse('login'))
        self.assertEqual(response.status_code, 200)
        site = get_current_site(response.request)
        self.assertEqual(response.context['site'], site)
        self.assertEqual(response.context['site_name'], site.name)
        self.assertTrue(isinstance(response.context['form'], AuthenticationForm),
                        'Login form is not an AuthenticationForm')

    def test_security_check(self, password='password'):
        login_url = reverse('login')

        # Those URLs should not pass the security check
        for bad_url in ('http://example.com',
                        'https://example.com',
                        'ftp://exampel.com',
                        '//example.com'):

            nasty_url = '%(url)s?%(next)s=%(bad_url)s' % {
                'url': login_url,
                'next': REDIRECT_FIELD_NAME,
                'bad_url': urlquote(bad_url),
            }
            response = self.client.post(nasty_url, {
                'username': self.default_login,
                'password': password,
            })
            self.assertEqual(response.status_code, 302)
            self.assertFalse(bad_url in response['Location'],
                             "%s should be blocked" % bad_url)

        # These URLs *should* still pass the security check
        for good_url in ('/view/?param=http://example.com',
                         '/view/?param=https://example.com',
                         '/view?param=ftp://exampel.com',
                         'view/?param=//example.com',
                         '//testserver/',
                         '/url%20with%20spaces/'):  # see ticket #12534
            safe_url = '%(url)s?%(next)s=%(good_url)s' % {
                'url': login_url,
                'next': REDIRECT_FIELD_NAME,
                'good_url': urlquote(good_url),
            }
            response = self.client.post(safe_url, {
                'username': self.default_login,
                'password': password,
            })
            self.assertEqual(response.status_code, 302)
            self.assertTrue(good_url in response['Location'],
                            "%s should be allowed" % good_url)

    def test_login_csrf_rotate(self, login=default_login, password='password'):
        """
        Makes sure that a login rotates the currently-used CSRF token.

        This is copy-pasted from django to allow specifying the login (username).
        """
        # Do a GET to establish a CSRF token
        # TestClient isn't used here as we're testing middleware, essentially.
        req = HttpRequest()
        CsrfViewMiddleware().process_view(req, login_view, (), {})
        # get_token() triggers CSRF token inclusion in the response
        get_token(req)
        resp = login_view(req)
        resp2 = CsrfViewMiddleware().process_response(req, resp)
        csrf_cookie = resp2.cookies.get(settings.CSRF_COOKIE_NAME, None)
        token1 = csrf_cookie.coded_value

        # Prepare the POST request
        req = HttpRequest()
        req.COOKIES[settings.CSRF_COOKIE_NAME] = token1
        req.method = "POST"
        req.POST = {'username': login, 'password': password, 'csrfmiddlewaretoken': token1}

        # Use POST request to log in
        SessionMiddleware().process_request(req)
        CsrfViewMiddleware().process_view(req, login_view, (), {})
        req.META["SERVER_NAME"] = "testserver"  # Required to have redirect work in login view
        req.META["SERVER_PORT"] = 80
        resp = login_view(req)
        resp2 = CsrfViewMiddleware().process_response(req, resp)
        csrf_cookie = resp2.cookies.get(settings.CSRF_COOKIE_NAME, None)
        token2 = csrf_cookie.coded_value

        # Check the CSRF token switched
        self.assertNotEqual(token1, token2)

    def test_login_form_contains_request(self):
        # The custom authentication form for this login requires a request to
        # initialize it.
        response = self.client.post('/custom_request_auth_login/', {
            'username': self.default_login,
            'password': 'password',
        })
        # The login was successful.
        self.assertRedirects(response, settings.LOGIN_REDIRECT_URL, fetch_redirect_response=False)

    if hasattr(LoginTest, 'test_security_check_https'):
        def test_security_check_https(self):
            login_url = reverse('login')
            non_https_next_url = 'http://testserver/path'
            not_secured_url = '%(url)s?%(next)s=%(next_url)s' % {
                'url': login_url,
                'next': REDIRECT_FIELD_NAME,
                'next_url': urlquote(non_https_next_url),
            }
            post_data = {
                'username': self.default_login,
                'password': 'password',
            }
            response = self.client.post(not_secured_url, post_data, secure=True)
            self.assertEqual(response.status_code, 302)
            self.assertNotEqual(response.url, non_https_next_url)
            self.assertEqual(response.url, settings.LOGIN_REDIRECT_URL)

    # these reference the builtin user model
    test_session_key_flushed_on_login_after_password_change = None
    test_login_session_without_hash_session_key = None


class DeprecationTest(WarningTestMixin, TestCase):

    def test_disallow_authenticated_is_deprecated_on_login_view(self):
        with self.assertWarns(DeprecationWarning):
            class CustomLoginView(LoginView):
                disallow_authenticated = False

            view = CustomLoginView()
            assert view.get_allow_authenticated()

        with self.assertWarns(DeprecationWarning):

            # Simulate LoginView.as_view(disallow_authenticated=False) behavior
            view = LoginView()
            view.disallow_authenticated = False

            assert view.get_allow_authenticated()


@override_settings(ROOT_URLCONF='tests.urls')
class LoginURLSettings(AuthViewsTestCaseDataMixin, LoginURLSettings):
    if DJANGO_VERSION[:2] < (1, 9):
        if settings.AUTH_USER_MODEL == 'authtools.User':
            fixtures = ['authtoolstestdata.json']
        elif settings.AUTH_USER_MODEL == 'tests.User':
            fixtures = ['customusertestdata.json']


@override_settings(ROOT_URLCONF='tests.urls')
class LogoutTest(EmailLoginMixin, AuthViewsTestCaseDataMixin, LogoutTest):
    test_logout_with_overridden_redirect_url = None
    test_logout_with_next_page_specified = None
    test_logout_with_custom_redirect_argument = None
    test_logout_with_named_redirect = None
    test_logout_with_custom_redirect_argument = None

    # the built-in tests depend on the django urlpatterns (they reverse
    # django.contrib.auth.views.login)
    def test_security_check(self, password='password'):
        logout_url = reverse('logout_then_login')

        # Those URLs should not pass the security check
        for bad_url in ('http://example.com',
                        'https://example.com',
                        'ftp://exampel.com',
                        '//example.com'):
            nasty_url = '%(url)s?%(next)s=%(bad_url)s' % {
                'url': logout_url,
                'next': REDIRECT_FIELD_NAME,
                'bad_url': urlquote(bad_url),
            }
            self.login()
            response = self.client.get(nasty_url)
            self.assertEqual(response.status_code, 302)
            self.assertFalse(bad_url in response['Location'],
                             "%s should be blocked" % bad_url)
            self.confirm_logged_out()

        # These URLs *should* still pass the security check
        for good_url in ('/view/?param=http://example.com',
                         '/view/?param=https://example.com',
                         '/view?param=ftp://exampel.com',
                         'view/?param=//example.com',
                         '//testserver/',
                         '/url%20with%20spaces/'):  # see ticket #12534
            safe_url = '%(url)s?%(next)s=%(good_url)s' % {
                'url': logout_url,
                'next': REDIRECT_FIELD_NAME,
                'good_url': urlquote(good_url),
            }
            self.login()
            response = self.client.get(safe_url)
            self.assertEqual(response.status_code, 302)
            self.assertTrue(good_url in response['Location'],
                            "%s should be allowed" % good_url)
            self.confirm_logged_out()


class UserCreationFormTest(TestCase):
    def setUp(self):
        # in built-in UserManager, the order of arguments is:
        #     username, email, password
        # in authtools UserManager, the order of arguments is:
        #     USERNAME_FIELD, password
        User.objects.create_user('testclient@example.com', password='test123')
        self.username = User.USERNAME_FIELD

    def test_user_already_exists(self):
        # The benefit of the custom validation message is only available if the
        # messages are translated.  We won't be able to translate all the
        # strings if we don't know what the username will be ahead of time.
        data = {
            self.username: 'testclient@example.com',
            'password1': 'test123',
            'password2': 'test123',
        }
        form = UserCreationForm(data)
        self.assertFalse(form.is_valid())
        self.assertEqual(form[self.username].errors, [
            force_text(form.error_messages['duplicate_username']) % {'username': self.username}])

    def test_password_verification(self):
        # The verification password is incorrect.
        data = {
            self.username: 'jsmith',
            'password1': 'test123',
            'password2': 'test',
        }
        form = UserCreationForm(data)
        self.assertFalse(form.is_valid())
        self.assertEqual(form["password2"].errors,
                         [force_text(form.error_messages['password_mismatch'])])

    def test_both_passwords(self):
        # One (or both) passwords weren't given
        data = {self.username: 'jsmith'}
        form = UserCreationForm(data)
        required_error = [force_text(Field.default_error_messages['required'])]
        self.assertFalse(form.is_valid())
        self.assertEqual(form['password1'].errors, required_error)
        self.assertEqual(form['password2'].errors, required_error)

        data['password2'] = 'test123'
        form = UserCreationForm(data)
        self.assertFalse(form.is_valid())
        self.assertEqual(form['password1'].errors, required_error)
        self.assertEqual(form['password2'].errors, [])

    def test_success(self):
        # The success case.
        data = {
            self.username: 'jsmith@example.com',
            'password1': 'test123',
            'password2': 'test123',
        }

        if settings.AUTH_USER_MODEL == 'authtools.User':
            data['name'] = 'John Smith'

        form = UserCreationForm(data)
        self.assertTrue(form.is_valid())
        u = form.save()
        self.assertEqual(getattr(u, self.username), 'jsmith@example.com')
        self.assertTrue(u.check_password('test123'))
        self.assertEqual(u, User._default_manager.get_by_natural_key('jsmith@example.com'))

    def test_generated_fields_list(self):
        if settings.AUTH_USER_MODEL == 'auth.User':
            fields = ('username', 'email', 'password1', 'password2')
        elif settings.AUTH_USER_MODEL == 'authtools.User':
            fields = ('email', 'name', 'password1', 'password2')
        elif settings.AUTH_USER_MODEL == 'tests.User':
            fields = ('email', 'full_name', 'preferred_name', 'password1', 'password2')
        else:
            assert False, "I don't know your user model"

        form = UserCreationForm()
        self.assertSequenceEqual(list(form.fields.keys()), fields)


@skipIfCustomUser
@override_settings(USE_TZ=False, PASSWORD_HASHERS=('django.contrib.auth.hashers.SHA1PasswordHasher',))
class UserChangeFormTest(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.u1 = User.objects.create(
            password='sha1$6efc0$f93efe9fd7542f25a7be94871ea45aa95de57161',
            last_login=datetime.datetime(2006, 12, 17, 7, 3, 31), is_superuser=False, username='testclient',
            first_name='Test', last_name='Client', email='testclient@example.com', is_staff=False, is_active=True,
            date_joined=datetime.datetime(2006, 12, 17, 7, 3, 31)
        )
        # cls.u3 = User.objects.create(
        #     password='sha1$6efc0$f93efe9fd7542f25a7be94871ea45aa95de57161',
        #     last_login=datetime.datetime(2006, 12, 17, 7, 3, 31), is_superuser=False, username='staff',
        #     first_name='Staff', last_name='Member', email='staffmember@example.com', is_staff=True, is_active=True,
        #     date_joined=datetime.datetime(2006, 12, 17, 7, 3, 31)
        # )
        cls.u4 = User.objects.create(
            password='', last_login=datetime.datetime(2006, 12, 17, 7, 3, 31), is_superuser=False,
            username='empty_password', first_name='Empty', last_name='Password', email='empty_password@example.com',
            is_staff=False, is_active=True, date_joined=datetime.datetime(2006, 12, 17, 7, 3, 31)
        )
        cls.u5 = User.objects.create(
            password='$', last_login=datetime.datetime(2006, 12, 17, 7, 3, 31), is_superuser=False,
            username='unmanageable_password', first_name='Unmanageable', last_name='Password',
            email='unmanageable_password@example.com', is_staff=False, is_active=True,
            date_joined=datetime.datetime(2006, 12, 17, 7, 3, 31)
        )
        cls.u6 = User.objects.create(
            password='foo$bar', last_login=datetime.datetime(2006, 12, 17, 7, 3, 31), is_superuser=False,
            username='unknown_password', first_name='Unknown', last_name='Password',
            email='unknown_password@example.com', is_staff=False, is_active=True,
            date_joined=datetime.datetime(2006, 12, 17, 7, 3, 31)
        )

    def test_bug_14242(self):
        # A regression test, introduce by adding an optimization for the
        # UserChangeForm.

        class MyUserForm(UserChangeForm):
            def __init__(self, *args, **kwargs):
                super(MyUserForm, self).__init__(*args, **kwargs)
                self.fields['groups'].help_text = 'These groups give users different permissions'

            class Meta(UserChangeForm.Meta):
                fields = ('groups',)

        # Just check we can create it
        MyUserForm({})

    def test_unsuable_password(self):
        user = User.objects.get(username='empty_password')
        user.set_unusable_password()
        user.save()
        form = UserChangeForm(instance=user)
        self.assertIn(_("No password set."), form.as_table())

    def test_bug_17944_empty_password(self):
        user = User.objects.get(username='empty_password')
        form = UserChangeForm(instance=user)
        self.assertIn(_("No password set."), form.as_table())

    def test_bug_17944_unmanageable_password(self):
        user = User.objects.get(username='unmanageable_password')
        form = UserChangeForm(instance=user)
        self.assertIn(_("Invalid password format or unknown hashing algorithm."),
                      form.as_table())

    def test_bug_17944_unknown_password_algorithm(self):
        user = User.objects.get(username='unknown_password')
        form = UserChangeForm(instance=user)
        self.assertIn(_("Invalid password format or unknown hashing algorithm."),
                      form.as_table())

    def test_bug_19133(self):
        "The change form does not return the password value"
        # Use the form to construct the POST data
        user = User.objects.get(username='testclient')
        form_for_data = UserChangeForm(instance=user)
        post_data = form_for_data.initial

        # The password field should be readonly, so anything
        # posted here should be ignored; the form will be
        # valid, and give back the 'initial' value for the
        # password field.
        post_data['password'] = 'new password'
        form = UserChangeForm(instance=user, data=post_data)

        self.assertTrue(form.is_valid())
        self.assertEqual(form.cleaned_data['password'], 'sha1$6efc0$f93efe9fd7542f25a7be94871ea45aa95de57161')

    def test_bug_19349_bound_password_field(self):
        user = User.objects.get(username='testclient')
        form = UserChangeForm(data={}, instance=user)
        # When rendering the bound password field,
        # ReadOnlyPasswordHashWidget needs the initial
        # value to render correctly
        self.assertEqual(form.initial['password'], form['password'].value())

    def test_better_readonly_password_widget(self):
        user = User.objects.get(username='testclient')
        form = UserChangeForm(instance=user)

        self.assertIn(_('*************'), form.as_table())


class UserAdminTest(TestCase):
    def test_generated_fieldsets(self):
        if settings.AUTH_USER_MODEL == 'auth.User':
            fields = ('username', 'email', 'password')
        elif settings.AUTH_USER_MODEL == 'authtools.User':
            fields = ('email', 'name', 'password')
        elif settings.AUTH_USER_MODEL == 'tests.User':
            fields = ('email', 'full_name', 'preferred_name', 'password')
        else:
            assert False, "I don't know your user model"

        self.assertSequenceEqual(BASE_FIELDS[1]['fields'], fields)


class UserManagerTest(TestCase):
    def test_create_user(self):
        u = User._default_manager.create_user(**{
            User.USERNAME_FIELD: 'newuser@example.com',
            'password': 'test123',
        })

        self.assertEqual(getattr(u, User.USERNAME_FIELD), 'newuser@example.com')
        self.assertTrue(u.check_password('test123'))
        self.assertEqual(u, User._default_manager.get_by_natural_key('newuser@example.com'))
        self.assertTrue(u.is_active)
        self.assertFalse(u.is_staff)
        self.assertFalse(u.is_superuser)

    @skipIfNotCustomUser
    def test_create_superuser(self):
        u = User._default_manager.create_superuser(**{
            User.USERNAME_FIELD: 'newuser@example.com',
            'password': 'test123',
        })

        self.assertTrue(u.is_staff)
        self.assertTrue(u.is_superuser)


class UserModelTest(TestCase):
    @skipUnless(settings.AUTH_USER_MODEL == 'authtools.User',
                         "only check authuser's ordering")
    def test_default_ordering(self):
        self.assertSequenceEqual(['name', 'email'], User._meta.ordering)

    def test_send_mail(self):
        abstract_user = User(email='foo@bar.com')
        abstract_user.email_user(subject="Subject here",
            message="This is a message", from_email="from@domain.com")
        # Test that one message has been sent.
        self.assertEqual(len(mail.outbox), 1)
        # Verify that test email contains the correct attributes:
        message = mail.outbox[0]
        self.assertEqual(message.subject, "Subject here")
        self.assertEqual(message.body, "This is a message")
        self.assertEqual(message.from_email, "from@domain.com")
        self.assertEqual(message.to, [abstract_user.email])


@override_settings(AUTHENTICATION_BACKENDS=['authtools.backends.CaseInsensitiveUsernameFieldModelBackend'])
class CaseInsensitiveTest(TestCase):
    form_class = CaseInsensitiveUsernameFieldCreationForm

    def get_form_data(self, data):
        base_data = {
            'auth.User': {},
            'authtools.User': {
                'name': 'Test Name',
            },
            'tests.User': {
                'full_name': 'Francis Underwood',
                'preferred_name': 'Frank',
            }
        }
        defaults = base_data[settings.AUTH_USER_MODEL]
        defaults.update(data)
        return defaults

    def test_case_insensitive_login_works(self):
        password = 'secret'
        form = self.form_class(self.get_form_data({
            User.USERNAME_FIELD: 'TEst@exAmPle.Com',
            'password1': password,
            'password2': password,
        }))
        self.assertTrue(form.is_valid(), form.errors)
        form.save()

        self.assertTrue(self.client.login(
            username='test@example.com',
            password=password,
        ))

        self.assertTrue(self.client.login(
            username='TEST@EXAMPLE.COM',
            password=password,
        ))


@override_settings(AUTHENTICATION_BACKENDS=['authtools.backends.CaseInsensitiveEmailModelBackend'])
class CaseInsensitiveAliasTest(TestCase):
    """Test that the aliases still work as well"""
    form_class = CaseInsensitiveEmailUserCreationForm

@override_settings(ROOT_URLCONF='tests.urls')
class LoginSuccessURLAllowedHostsTest(AuthViewsTestCaseDataMixin, LoginSuccessURLAllowedHostsTest):
    if settings.AUTH_USER_MODEL == 'auth.User':
        default_login = 'testclient'
    else:
        default_login = 'testclient@example.com'

    def test_success_url_allowed_hosts_same_host(self):
        response = self.client.post('/login/allowed_hosts/', {
            'username': self.default_login,
            'password': 'password',
            'next': 'https://testserver/home',
        })
        self.assertIn(SESSION_KEY, self.client.session)
        self.assertRedirects(response, 'https://testserver/home', fetch_redirect_response=False)

    def test_success_url_allowed_hosts_safe_host(self):
        response = self.client.post('/login/allowed_hosts/', {
            'username': self.default_login,
            'password': 'password',
            'next': 'https://otherserver/home',
        })
        self.assertIn(SESSION_KEY, self.client.session)
        self.assertRedirects(response, 'https://otherserver/home', fetch_redirect_response=False)

    def test_success_url_allowed_hosts_unsafe_host(self):
        response = self.client.post('/login/allowed_hosts/', {
            'username': self.default_login,
            'password': 'password',
            'next': 'https://evil/home',
        })
        self.assertIn(SESSION_KEY, self.client.session)
        self.assertRedirects(response, '/accounts/profile/', fetch_redirect_response=False)



from django.contrib.auth.backends import ModelBackend
from django import VERSION as DJANGO_VERSION


class CaseInsensitiveUsernameFieldBackendMixin(object):
    """
    This authentication backend assumes that usernames are email addresses and simply
    lowercases a username before an attempt is made to authenticate said username using a
    superclass's authenticate method. This superclass should be either a user-defined
    authentication backend, or a Django-provided authentication backend (e.g., ModelBackend).

    Example usage:
        See CaseInsensitiveUsernameFieldBackend, below.

    NOTE:
        A word of caution. Use of this backend presupposes a way to ensure that users cannot
        create usernames that differ only in case (e.g., joe@test.org and JOE@test.org). It is
        advised that you use this backend in conjunction with the
        CaseInsensitiveUsernameFieldCreationForm provided in the forms module.
    """

    def authenticate(self, request, username=None, password=None, **kwargs):
        if username is not None:
            username = username.lower()

        return super(CaseInsensitiveUsernameFieldBackendMixin, self).authenticate(
            request,
            username=username,
            password=password,
            **kwargs
        )

class CaseInsensitiveUsernameFieldModelBackend(
        CaseInsensitiveUsernameFieldBackendMixin,
        ModelBackend):
    pass


# alias for the old name for backwards-compatability
CaseInsensitiveEmailBackendMixin = CaseInsensitiveUsernameFieldBackendMixin
CaseInsensitiveEmailModelBackend = CaseInsensitiveUsernameFieldModelBackend



from django.contrib import admin
from django import forms
from django.contrib.auth import get_user_model
from django.contrib.auth.forms import PasswordResetForm
from django.utils.crypto import get_random_string

from authtools.admin import NamedUserAdmin
from authtools.forms import UserCreationForm

User = get_user_model()


class UserCreationForm(UserCreationForm):
    """
    A UserCreationForm with optional password inputs.
    """

    def __init__(self, *args, **kwargs):
        super(UserCreationForm, self).__init__(*args, **kwargs)
        self.fields['password1'].required = False
        self.fields['password2'].required = False
        # If one field gets autocompleted but not the other, our 'neither
        # password or both password' validation will be triggered.
        self.fields['password1'].widget.attrs['autocomplete'] = 'off'
        self.fields['password2'].widget.attrs['autocomplete'] = 'off'

    def clean_password2(self):
        password1 = self.cleaned_data.get("password1")
        password2 = super(UserCreationForm, self).clean_password2()
        if bool(password1) ^ bool(password2):
            raise forms.ValidationError("Fill out both fields")
        return password2


class UserAdmin(NamedUserAdmin):
    """
    A UserAdmin that sends a password-reset email when creating a new user,
    unless a password was entered.
    """
    add_form = UserCreationForm
    add_fieldsets = (
        (None, {
            'description': (
                "Enter the new user's name and email address and click save."
                " The user will be emailed a link allowing them to login to"
                " the site and set their password."
            ),
            'fields': ('email', 'name',),
        }),
        ('Password', {
            'description': "Optionally, you may set the user's password here.",
            'fields': ('password1', 'password2'),
            'classes': ('collapse', 'collapse-closed'),
        }),
    )

    def save_model(self, request, obj, form, change):
        if not change and not obj.has_usable_password():
            # Django's PasswordResetForm won't let us reset an unusable
            # password. We set it above super() so we don't have to save twice.
            obj.set_password(get_random_string())
            reset_password = True
        else:
            reset_password = False

        super(UserAdmin, self).save_model(request, obj, form, change)

        if reset_password:
            reset_form = PasswordResetForm({'email': obj.email})
            assert reset_form.is_valid()
            reset_form.save(
                subject_template_name='registration/account_creation_subject.txt',
                email_template_name='registration/account_creation_email.html',
            )

admin.site.unregister(User)
admin.site.register(User, UserAdmin)



from django.db import models

from authtools.models import AbstractEmailUser


class User(AbstractEmailUser):
    full_name = models.CharField('full name', max_length=255, blank=True)
    preferred_name = models.CharField('preferred name', max_length=255,
                                      blank=True)

    REQUIRED_FIELDS = ['full_name', 'preferred_name']

    class Meta:
        # We need this line to prevent manage.py validate clashes.
        swappable = 'AUTH_USER_MODEL'

    def get_full_name(self):
        return self.full_name

    def get_short_name(self):
        return self.preferred_name



#!/usr/bin/env python
import io
import os

from setuptools import setup, find_packages

__doc__ = ("Custom user model app for Django featuring email as username and"
           " class-based views for authentication.")


def read(fname):
    return io.open(os.path.join(os.path.dirname(__file__), fname), encoding="utf-8").read()


install_requires = [
    'Django>=1.11',
]

setup(
    name='olivia-django-authtools',
    version='1.0.0',
    author='phongtran',
    author_email='phong.tran@paradox.ai',
    license="MIT",
    description=__doc__,
    long_description=__doc__,
    long_description_content_type="text/x-rst",
    url='https://github.com/nguyencg/django-authtools.git',
    packages=[package for package in find_packages() if package.startswith('authtools')],
    install_requires=install_requires,
    include_package_data=True,
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
    ],
)



from django.conf.urls import include, url
from django.http import HttpResponse
from django.contrib.auth.decorators import login_required
from django.contrib import admin

from django.urls import reverse_lazy

from authtools import views
from authtools.forms import FriendlyPasswordResetForm

from auth_tests.urls import CustomRequestAuthenticationForm, uid_token

admin.autodiscover()


def dumbview(request):
    return HttpResponse('dumbview')


urlpatterns = [
    url(r'^reset_and_login/(?P<uidb64>[0-9A-Za-z_\-]+)/(?P<token>[0-9A-Za-z]{1,13}-[0-9A-Za-z]{1,20})/$', views.PasswordResetConfirmAndLoginView.as_view()),
    url(r'^logout-then-login/$', views.LogoutView.as_view(url=reverse_lazy('login')), name='logout_then_login'),
    url(r'^friendly_password_reset/$',
        views.PasswordResetView.as_view(form_class=FriendlyPasswordResetForm),
        name='friendly_password_reset'),
    url(r'^login_required/$', login_required(dumbview), name='login_required'),
    # From django.contrib.auth.tests.url

    url(r'^password_reset_extra_email_context/$', views.PasswordResetView.as_view(extra_email_context=dict(greeting='Hello!'))),
    url(r'^password_reset/html_email_template/$', views.PasswordResetView.as_view(html_email_template_name='registration/html_password_reset_email.html')),
    url(r'^', include('authtools.urls')),
]

urlpatterns += [
    url(r'^reset/post_reset_login/{}/$'.format(uid_token),
        views.PasswordResetConfirmView.as_view(post_reset_login=True)),
    url(r'^custom_request_auth_login/$',
        views.LoginView.as_view(authentication_form=CustomRequestAuthenticationForm)),
    url(
        r'^reset/post_reset_login_custom_backend/{}/$'.format(uid_token),
        views.PasswordResetConfirmView.as_view(
            post_reset_login=True,
            post_reset_login_backend='django.contrib.auth.backends.AllowAllUsersModelBackend',
        ),
    ),
    url(r'^logout/allowed_hosts/$', views.LogoutView.as_view(success_url_allowed_hosts={'otherserver'})),
    url(r'^login/allowed_hosts/$', views.LoginView.as_view(success_url_allowed_hosts={'otherserver'})),
]



from __future__ import unicode_literals

import copy

from django.contrib import admin
from django.contrib.auth.admin import UserAdmin as DjangoUserAdmin
from django.contrib.auth import get_user_model
from django.contrib.auth.forms import UserCreationForm
from django.utils.translation import ugettext_lazy as _

from authtools.forms import UserChangeForm
from authtools.models import User

USERNAME_FIELD = get_user_model().USERNAME_FIELD

REQUIRED_FIELDS = (USERNAME_FIELD,) + tuple(get_user_model().REQUIRED_FIELDS)

BASE_FIELDS = (None, {
    'fields': REQUIRED_FIELDS + ('password',),
})

SIMPLE_PERMISSION_FIELDS = (_('Permissions'), {
    'fields': ('is_active', 'is_staff', 'is_superuser',),
})

ADVANCED_PERMISSION_FIELDS = copy.deepcopy(SIMPLE_PERMISSION_FIELDS)
ADVANCED_PERMISSION_FIELDS[1]['fields'] += ('groups', 'user_permissions',)

DATE_FIELDS = (_('Important dates'), {
    'fields': ('last_login', 'date_joined',),
})


class StrippedUserAdmin(DjangoUserAdmin):
    # The forms to add and change user instances
    add_form_template = None
    add_form = UserCreationForm
    form = UserChangeForm

    # The fields to be used in displaying the User model.
    # These override the definitions on the base UserAdmin
    # that reference specific fields on auth.User.
    list_display = ('is_active', USERNAME_FIELD, 'is_superuser', 'is_staff',)
    list_display_links = (USERNAME_FIELD,)
    list_filter = ('is_superuser', 'is_staff', 'is_active',)
    fieldsets = (
        BASE_FIELDS,
        SIMPLE_PERMISSION_FIELDS,
    )
    add_fieldsets = (
        (None, {
            'fields': REQUIRED_FIELDS + (
                'password1',
                'password2',
            ),
        }),
    )
    search_fields = (USERNAME_FIELD,)
    ordering = None
    filter_horizontal = tuple()
    readonly_fields = ('last_login', 'date_joined')


class StrippedNamedUserAdmin(StrippedUserAdmin):
    list_display = ('is_active', 'email', 'name', 'is_superuser', 'is_staff',)
    list_display_links = ('email', 'name',)
    search_fields = ('email', 'name',)


class UserAdmin(StrippedUserAdmin):
    fieldsets = (
        BASE_FIELDS,
        ADVANCED_PERMISSION_FIELDS,
        DATE_FIELDS,
    )
    filter_horizontal = ('groups', 'user_permissions',)


class NamedUserAdmin(UserAdmin, StrippedNamedUserAdmin):
    pass


# If the model has been swapped, this is basically a noop.
admin.site.register(User, NamedUserAdmin)



from __future__ import unicode_literals

from django import forms, VERSION as DJANGO_VERSION
from django.forms.utils import flatatt
from django.contrib.auth.forms import (
    ReadOnlyPasswordHashField, ReadOnlyPasswordHashWidget,
    PasswordResetForm as OldPasswordResetForm,
    UserChangeForm as DjangoUserChangeForm,
    AuthenticationForm as DjangoAuthenticationForm,
)
from django.contrib.auth import get_user_model, password_validation
from django.contrib.auth.hashers import identify_hasher, UNUSABLE_PASSWORD_PREFIX
from django.utils.translation import ugettext_lazy as _, ugettext
from django.utils.html import format_html

User = get_user_model()


def is_password_usable(pw):
    """Decide whether a password is usable only by the unusable password prefix.

    We can't use django.contrib.auth.hashers.is_password_usable either, because
    it not only checks against the unusable password, but checks for a valid
    hasher too. We need different error messages in those cases.
    """

    return not pw.startswith(UNUSABLE_PASSWORD_PREFIX)


class BetterReadOnlyPasswordHashWidget(ReadOnlyPasswordHashWidget):
    """
    A ReadOnlyPasswordHashWidget that has a less intimidating output.
    """
    def render(self, name, value, attrs=None, renderer=None):
        final_attrs = flatatt(self.build_attrs(attrs or {}))

        if not value or not is_password_usable(value):
            summary = ugettext("No password set.")
        else:
            try:
                identify_hasher(value)
            except ValueError:
                summary = ugettext("Invalid password format or unknown"
                                   " hashing algorithm.")
            else:
                summary = ugettext('*************')

        return format_html('<div{attrs}><strong>{summary}</strong></div>',
                           attrs=final_attrs, summary=summary)


class UserChangeForm(DjangoUserChangeForm):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        password = self.fields.get('password')
        if password:
            password.widget = BetterReadOnlyPasswordHashWidget()


class UserCreationForm(forms.ModelForm):
    """
    A form for creating new users. Includes all the required
    fields, plus a repeated password.
    """

    error_messages = {
        'password_mismatch': _("The two password fields didn't match."),
        'duplicate_username': _("A user with that %(username)s already exists."),
    }

    password1 = forms.CharField(label=_("Password"), widget=forms.PasswordInput)
    password2 = forms.CharField(label=_("Password confirmation"),
                                widget=forms.PasswordInput,
                                help_text=_("Enter the same password as above,"
                                            " for verification."))

    class Meta:
        model = User
        fields = (User.USERNAME_FIELD,) + tuple(User.REQUIRED_FIELDS)

    def __init__(self, *args, **kwargs):
        super(UserCreationForm, self).__init__(*args, **kwargs)

        def validate_uniqueness_of_username_field(value):
            # Since User.username is unique, this check is redundant,
            # but it sets a nicer error message than the ORM. See #13147.
            try:
                User._default_manager.get_by_natural_key(value)
            except User.DoesNotExist:
                return value
            raise forms.ValidationError(self.error_messages['duplicate_username'] % {
                'username': User.USERNAME_FIELD,
            })

        self.fields[User.USERNAME_FIELD].validators.append(validate_uniqueness_of_username_field)

    def clean_password2(self):
        # Check that the two password entries match
        password1 = self.cleaned_data.get("password1")
        password2 = self.cleaned_data.get("password2")
        if password1 and password2 and password1 != password2:
            raise forms.ValidationError(self.error_messages['password_mismatch'])
        return password2

    def _post_clean(self):
        super(UserCreationForm, self)._post_clean()
        # Validate the password after self.instance is updated with form data
        # by super().
        password = self.cleaned_data.get('password2')
        if password:
            try:
                password_validation.validate_password(password, self.instance)
            except forms.ValidationError as error:
                self.add_error('password2', error)

    def save(self, commit=True):
        # Save the provided password in hashed format
        user = super(UserCreationForm, self).save(commit=False)
        user.set_password(self.cleaned_data["password1"])
        if commit:
            user.save()
        return user


class CaseInsensitiveUsernameFieldCreationForm(UserCreationForm):
    """
    This form is the same as UserCreationForm, except that usernames are lowercased before they
    are saved. This is to disallow the existence of email address usernames which differ only in
    case.
    """
    def clean_USERNAME_FIELD(self):
        username = self.cleaned_data.get(User.USERNAME_FIELD)
        if username:
            username = username.lower()

        return username

# set the correct clean method on the class so that child classes can override and call super()
setattr(
    CaseInsensitiveUsernameFieldCreationForm,
    'clean_' + User.USERNAME_FIELD,
    CaseInsensitiveUsernameFieldCreationForm.clean_USERNAME_FIELD
)

# alias for the old name for backwards-compatability
CaseInsensitiveEmailUserCreationForm = CaseInsensitiveUsernameFieldCreationForm


class FriendlyPasswordResetForm(OldPasswordResetForm):
    error_messages = dict(getattr(OldPasswordResetForm, 'error_messages', {}))
    error_messages['unknown'] = _("This email address doesn't have an "
                                  "associated user account. Are you "
                                  "sure you've registered?")

    def clean_email(self):
        """Return an error message if the email address being reset is unknown.

        This is to revert https://code.djangoproject.com/ticket/19758
        The bug #19758 tries not to leak emails through password reset because
        only usernames are unique in Django's default user model.

        django-authtools leaks email addresses through the registration form.
        In the case of django-authtools not warning the user doesn't add any
        security, and worsen user experience.
        """

        email = self.cleaned_data['email']
        results = list(self.get_users(email))

        if not results:
            raise forms.ValidationError(self.error_messages['unknown'])
        return email


class AuthenticationForm(DjangoAuthenticationForm):
    def __init__(self, request=None, *args, **kwargs):
        super(AuthenticationForm, self).__init__(request, *args, **kwargs)
        username_field = User._meta.get_field(User.USERNAME_FIELD)
        self.fields['username'].widget = username_field.formfield().widget
